"use strict";

/**
 * Class representing reaction data.
 * 
 * @param javaObject
 *          object deserialized from ajax query to the server side
 */
function Reaction(javaObject) {
  this.startLines = [];
  this.endLines = [];
  this.midLines = [];
  if (javaObject instanceof Reaction) {
    this.setId(javaObject.getId());
    this.setCenter(javaObject.getCenter());
    this.setModelId(javaObject.getModelId());

    this.startLines.push.apply(this.startLines, javaObject.getStartLines());
    this.endLines.push.apply(this.endLines, javaObject.getEndLines());
    this.midLines.push.apply(this.midLines, javaObject.getMidLines());

  } else {
    if (javaObject.idObject !== undefined) {
      this.setId(javaObject.idObject);
    } else {
      this.setId(javaObject.id);
    }
    for (var i = 0; i < javaObject.lines.length; i++) {
      var line = javaObject.lines[i];
      if (line.type === "START") {
        this.startLines.push(line);
      } else if (line.type === "END") {
        this.endLines.push(line);
      } else if (line.type === "MIDDLE") {
        this.midLines.push(line);
      } else {
        throw new Error("Unknown line type: " + line.type);
      }
    }
    this.setCenter(javaObject.centerPoint);
    this.setModelId(javaObject.modelId);
    this.setIsComplete(false);
    this.update(javaObject);
  }
}

Reaction.prototype.getId = function() {
  return this.id;
};

Reaction.prototype.setId = function(id) {
  this.id = parseInt(id);
};

Reaction.prototype.getCenter = function() {
  return this._center;
};

Reaction.prototype.getLines = function() {
  var result = [];
  result = result.concat(this.startLines);
  result = result.concat(this.endLines);
  result = result.concat(this.midLines);
  return result;
};

Reaction.prototype.getMidLines = function() {
  return this.midLines;
};
Reaction.prototype.getStartLines = function() {
  return this.startLines;
};
Reaction.prototype.getEndLines = function() {
  return this.endLines;
};

Reaction.prototype.setCenter = function(center) {
  if (center === null || center === undefined) {
    throw new Error("Setting undefined center: ", center);
  }
  this._center = center;
};

Reaction.prototype.getModelId = function() {
  return this._modelId;
};

Reaction.prototype.setModelId = function(modelId) {
  this._modelId = modelId;
};

Reaction.prototype.update = function(javaObject) {
  if (javaObject.reactionId === undefined) {
    return;
  }
  this.setReactionId(javaObject.reactionId);
  this.setSymbol(javaObject.symbol);
  this.setAbbreviation(javaObject.abbreviation);
  this.setFormula(javaObject.formula);
  this.setMechanicalConfidenceScore(javaObject.mechanicalConfidenceScore);
  this.setLowerBound(javaObject.lowerBound);
  this.setUpperBound(javaObject.upperBound);
  this.setGeneProteinReaction(javaObject.geneProteinReaction);
  this.setSubsystem(javaObject.subsystem);
  this.setSynonyms(javaObject.synonyms);
  this.setDescription(javaObject.notes);
  this.setOther(javaObject.other);
  this.setReferences(javaObject.references);
  
  if (javaObject.reactants !== "") {
    this.setReactants(javaObject.reactants.split(","));
  } else {
    this.setReactants([]);
  }
  if (javaObject.products !== "") {
    this.setProducts(javaObject.products.split(","));
  } else {
    this.setProducts([]);
  }
  if (javaObject.modifiers !== "") {
    this.setModifiers(javaObject.modifiers.split(","));
  } else {
    this.setModifiers([]);
  }
  
  this.setIsComplete(true);
};

Reaction.prototype.isComplete = function() {
  return this._complete;
};

Reaction.prototype.setIsComplete = function(complete) {
  this._complete = complete;
};

Reaction.prototype.getReactionId = function() {
  return this._reactionId;
};

Reaction.prototype.setReactionId = function(reactionId) {
  this._reactionId = reactionId;
};

Reaction.prototype.getSymbol = function() {
  return this._symbol;
};

Reaction.prototype.setSymbol = function(symbol) {
  this._symbol = symbol;
};

Reaction.prototype.getAbbreviation= function() {
  return this._abbreviation;
};

Reaction.prototype.setAbbreviation= function(abbreviation) {
  this._abbreviation = abbreviation;
};

Reaction.prototype.getFormula= function() {
  return this._formula;
};

Reaction.prototype.setFormula = function(formula) {
  this._formula = formula;
};

Reaction.prototype.getMechanicalConfidenceScore= function() {
  return this._mechanicalConfidenceScore;
};

Reaction.prototype.setMechanicalConfidenceScore = function(mechanicalConfidenceScore) {
  this._mechanicalConfidenceScore = mechanicalConfidenceScore;
};

Reaction.prototype.getLowerBound= function() {
  return this._lowerBound;
};

Reaction.prototype.setLowerBound = function(lowerBound) {
  this._lowerBound = lowerBound;
};

Reaction.prototype.getUpperBound= function() {
  return this._upperBound;
};

Reaction.prototype.setUpperBound = function(upperBound) {
  this._upperBound = upperBound;
};

Reaction.prototype.setGeneProteinReaction = function(geneProteinReaction) {
  this._geneProteinReaction = geneProteinReaction;
};

Reaction.prototype.getGeneProteinReaction= function() {
  return this._geneProteinReaction;
};

Reaction.prototype.setSubsystem = function(subsystem) {
  this._subsystem = subsystem;
};

Reaction.prototype.getSubsystem= function() {
  return this._subsystem;
};

Reaction.prototype.setSynonyms = function(synonyms) {
  this._synonyms = synonyms;
};

Reaction.prototype.getSynonyms= function() {
  return this._synonyms;
};

Reaction.prototype.setDescription = function(description) {
  this._description = description;
};

Reaction.prototype.getDescription= function() {
  return this._description;
};

Reaction.prototype.getReactants = function() {
  return this._reactants;
};

Reaction.prototype.setReactants = function(reactants) {
  this._reactants = reactants;
};

Reaction.prototype.setProducts = function(products) {
  this._products = products;
};

Reaction.prototype.getProducts = function() {
  return this._products;
};

Reaction.prototype.getElements = function() {
  var result = [];
  result = result.concat(this.getReactants());
  result = result.concat(this.getProducts());
  result = result.concat(this.getModifiers());
  return result;
};

Reaction.prototype.setModifiers = function(modifiers) {
  this._modifiers = modifiers;
};

Reaction.prototype.getModifiers = function() {
  return this._modifiers;
};

Reaction.prototype.getOther = function(type) {
  if (this._other !== undefined) {
    return this._other[type];
  }
};

Reaction.prototype.setOther = function(other) {
  this._other = other;
};

Reaction.prototype.getReferences = function() {
  return this.references;
};

Reaction.prototype.setReferences = function(references) {
  this.references = references;
};

module.exports = Reaction;
