"use strict";

var AbstractMarker = require('./AbstractMarker');

/**
 * This class describes Google maps marker (connected to {@link Reaction}) that
 * is visualized on the map.
 */
function ReactionMarker(id, icon, reactionData, map) {
  AbstractMarker.call(this, icon, map);
  // id of the reaction to which marker is connected
  this._id = id;
  // data about the Reaction connected to the marker
  this._reactionData = reactionData;
  // google.maps.Marker object
  this._marker = null;
  if (this._reactionData !== null && this._reactionData !== undefined) {
    this.init();
    this.show();
  }
}
ReactionMarker.prototype = Object.create(AbstractMarker.prototype);
ReactionMarker.prototype.constructor = ReactionMarker;

/**
 * Returns identifier of this object.
 * 
 * @returns identifier of this object
 */
ReactionMarker.prototype.getId = function() {
  return this._id;
};

/**
 * Returns {@link Reaction} data for this marker.
 * 
 * @returns {@link Reaction} data for this marker
 */
ReactionMarker.prototype.getReactionData = function() {
  return this._reactionData;
};

/**
 * Sets {@link Reaction} data for this marker.
 * 
 * @param data
 *            {@link Reaction} data for this marker
 */
ReactionMarker.prototype.setReactionData = function(data) {
  this._reactionData = data;
};

/**
 * Returns coordinates where marker is poiting.
 * 
 * @returns {google.maps.Point} - coordinates where marker is poiting
 */
ReactionMarker.prototype.getCoordinates = function() {
  return new google.maps.Point(this._reactionData.getCenter().x,
      this._reactionData.getCenter().y);
};

/**
 * Function returning string marker type.
 * 
 * @returns string marker type
 */
ReactionMarker.prototype.getType = function() {
  return "REACTION";
};

module.exports = ReactionMarker;
