"use strict";

/* exported logger */

var logger = require('../../logger');

var Promise = require("bluebird");

var IdentifiedElement = require('../data/IdentifiedElement');
var AbstractDbOverlay = require('./AbstractDbOverlay');

function AbstractTargettingDbOverlay(params) {
  // call super constructor
  AbstractDbOverlay.call(this, params);
}

AbstractTargettingDbOverlay.prototype = Object.create(AbstractDbOverlay.prototype);
AbstractTargettingDbOverlay.prototype.constructor = AbstractTargettingDbOverlay;

AbstractTargettingDbOverlay.prototype.getElementsByQuery = function(query) {
  var self = this;
  return new Promise(function(resolve){
    var i;
    var queryId = null;
    var queries =self.getQueries(); 
    for (i=0;i<queries.length;i++) {
      if (queries[i]===query) {
        queryId = i;
      }
    }
    if (queryId===null) {
      throw new Error("Invalid query: "+query);
    }
    var colourId = queryId;
    var elements = self._elementsByQuery[query];

    var result = [];
    if (elements.length>0) {
      if (elements.length>1) {
        logger.warn("More than one element per query not implemented");
      }
      var element = elements[0];
      result.element = element;
      var iconCounter = 1;
      for (i=0; i<element.getTargets().length; i++) {
        var target = element.getTargets()[i];
        var icon = self.getIcon(colourId, iconCounter++);
        if (target.getTargetElements().length===0) {
          icon = null;
        }
        result.push({element:target, icon:icon});
      }
    }
    resolve(result);
  });
};

AbstractTargettingDbOverlay.prototype.searchBySingleQuery = function(originalQuery) {
  var self = this;
  return new Promise(function(resolve, reject) {
    var query = self.encodeQuery(AbstractDbOverlay.QueryType.SEARCH_BY_QUERY, originalQuery);
    if (self._elementsByQuery[query] !== undefined) {
      resolve(self._elementsByQuery[query]);
    } else {
      return self.getElementsByQueryFromServer({
        query:originalQuery
      }).then(function(elements) {
        self._elementsByQuery[query] = elements;
        
        var identifiedElements = [];
        for (var i=0;i<elements.length;i++) {
          var targets = elements[i].getTargets();
          for (var j=0;j<targets.length;j++) {
            var target = targets[j];
            for (var k=0;k<target.targetElements;k++) {
              identifiedElements.push(new IdentifiedElement(target.targetElements[i]));
            }
          }
        }
        return self.getMap().fetchIdentifiedElements(identifiedElements, true);
      }).then(function(){
        resolve(self._elementsByQuery[query]);
      }).catch(reject);
    }
  });
};

AbstractTargettingDbOverlay.prototype.getIdentifiedElements = function(){
  var self = this;

  return new Promise(function(resolve){
    var queries = self.getQueries();
    var result = [];
    var colourId = 0;
    for (var i=0;i<queries.length;i++) {
      var query = queries[i];
      var elements = self._elementsByQuery[query];

      for (var j=0;j<elements.length;j++) {
        var element = elements[j];
        
        var targetElements = self.createIdentifiedElementsForTargetingClass(element, colourId);
        
        result.push.apply(result, targetElements);
        
        colourId++;
      }
    }
    resolve(result);
  });
};

AbstractTargettingDbOverlay.prototype.getDetailDataByIdentifiedElement = function(element, general) {
  var self = this;
  if (general) {
    return self.searchNamesByTarget(element);
  } else {
    return new Promise(function(resolve){
      var result =[];
      var queries =self.getQueries();
      for (var i=0;i<queries.length;i++) {
        var drugs = self._elementsByQuery[queries[i]];
        if (drugs===undefined) {
          drugs = [];
        }
        for (var j=0;j<drugs.length;j++){
          var drug = drugs[j];
          var targets = drug.getTargets();
          for (var k=0;k<targets.length;k++) {
            var elements = targets[k].getTargetElements();
            for (var l=0;l<elements.length;l++) {
              if (element.equals(elements[l])) {
                result.push(drug.getName());
              }
            }
          }
        }
      }
      resolve(result);
    });
  }
};

module.exports = AbstractTargettingDbOverlay;
